﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
//using System.Threading.Tasks;
using System.Windows.Forms;
using Phidget22;
using Phidget22.Events;
using Phidget22.ExampleUtils;

namespace DistanceSensor_Example
{
    public partial class Form1 : Form
    {
        CommandLineOpen open;
        DistanceSensor distance = null;
        private ErrorEventBox errorBox;
        formCleaner cleaner;

        public Form1()
        {
            open = new CommandLineOpen(this);
            cleaner = new formCleaner(this);
            InitializeComponent();
        }

        public Form1(String[] commandLine)
        {
            open = new CommandLineOpen(this);
            cleaner = new formCleaner(this);
            open.commandLine = commandLine;
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            phidgetInfoBox1.Clear();

            
            errorBox = new ErrorEventBox(this);
            distance = new DistanceSensor();
            
            distance.Attach += new Phidget22.Events.AttachEventHandler(distance_attach);
            distance.Detach += new Phidget22.Events.DetachEventHandler(distance_detach);
            distance.Error += new Phidget22.Events.ErrorEventHandler(distance_error);
            distance.SonarReflectionsUpdate += new Phidget22.Events.DistanceSensorSonarReflectionsUpdateEventHandler(distance_update);
            distance.DistanceChange += new Phidget22.Events.DistanceSensorDistanceChangeEventHandler(distance_change);

            open.openCmdLine(distance);
        }

        private void Form1_FormClosing(object sender, FormClosingEventArgs e)
        {
            distance.Attach -= new Phidget22.Events.AttachEventHandler(distance_attach);
            distance.Detach -= new Phidget22.Events.DetachEventHandler(distance_detach);
            distance.Error -= new Phidget22.Events.ErrorEventHandler(distance_error);
            distance.SonarReflectionsUpdate -= new Phidget22.Events.DistanceSensorSonarReflectionsUpdateEventHandler(distance_update);
            Application.DoEvents();
            distance.Close();
        }

        void distance_attach(object sender, Phidget22.Events.AttachEventArgs e)
        {
            phidgetInfoBox1.FillPhidgetInfo((Phidget)sender);

            DistanceSensor attachedDevice = (DistanceSensor)sender;

            try
            {//configure form controls
                dataIntervalTrk.Unit = " ms";
                dataIntervalTrk.minTxt = attachedDevice.MinDataInterval.ToString();
                dataIntervalTrk.maxTxt = attachedDevice.MaxDataInterval.ToString();
                dataIntervalTrk.Minimum = attachedDevice.MinDataInterval;
                dataIntervalTrk.Maximum = attachedDevice.MaxDataInterval;
                dataIntervalTrk.Value = attachedDevice.DataInterval;

                //differing max/min means the interval is a settable configuration
                if (attachedDevice.MaxDataInterval == attachedDevice.MinDataInterval)
                    dataIntervalTrk.Enabled = false;



                changeTriggerTrk.Unit = "mm";
                changeTriggerTrk.minTxt = attachedDevice.MinDistanceChangeTrigger.ToString();
                changeTriggerTrk.maxTxt = attachedDevice.MaxDistanceChangeTrigger.ToString();
                changeTriggerTrk.Minimum = attachedDevice.MinDistanceChangeTrigger;
                changeTriggerTrk.Maximum = attachedDevice.MaxDistanceChangeTrigger;
                changeTriggerTrk.Value = attachedDevice.DistanceChangeTrigger;

                if (attachedDevice.DeviceID == DeviceID.PN_DST1200)
                {
                    attachedDevice.SonarQuietMode = true;
                    sonarQuietModeBox.Visible = true;
                    sonarQuietModeBox.Checked = true;
                    sonarReflectionsBox.Visible = true;
                }
            }
            catch (PhidgetException ex)
            { 
                errorBox.addMessage("Error setting up sonar sensor: " + ex.Message); 
            }

            settingsBox.Visible = true;
            dataBox.Visible = true;

        }
        void distance_detach(object sender, Phidget22.Events.DetachEventArgs e)
        {
            phidgetInfoBox1.Clear();
            cleaner.clean();

            settingsBox.Visible = false;
            sonarQuietModeBox.Visible = false;
            sonarReflectionsBox.Visible = false;
            dataBox.Visible = false;
        }
        void distance_error(object sender, Phidget22.Events.ErrorEventArgs e)
        {
            errorBox.addMessage(e.Description);
        }

         void distance_change(object sender, Phidget22.Events.DistanceSensorDistanceChangeEventArgs e)
        {
            try
            {
                valueTxt.Text = e.Distance.ToString() + "mm";
            }
            catch (PhidgetException ex)
            { errorBox.addMessage(ex.ToString()); }
        }

        void distance_update(object sender, Phidget22.Events.DistanceSensorSonarReflectionsUpdateEventArgs e)
        {
            try
            {
                countTxt.Text = e.Count.ToString();

                distance0.Text = (0 < e.Count) ? e.Distances[0].ToString() : "";
                distance1.Text = (1 < e.Count) ? e.Distances[1].ToString() : "";
                distance2.Text = (2 < e.Count) ? e.Distances[2].ToString() : "";
                distance3.Text = (3 < e.Count) ? e.Distances[3].ToString() : "";
                distance4.Text = (4 < e.Count) ? e.Distances[4].ToString() : "";
                distance5.Text = (5 < e.Count) ? e.Distances[5].ToString() : "";
                distance6.Text = (6 < e.Count) ? e.Distances[6].ToString() : "";
                distance7.Text = (7 < e.Count) ? e.Distances[7].ToString() : "";

                amplitude0.Text = (0 < e.Count) ? e.Amplitudes[0].ToString() : "";
                amplitude1.Text = (1 < e.Count) ? e.Amplitudes[1].ToString() : "";
                amplitude2.Text = (2 < e.Count) ? e.Amplitudes[2].ToString() : "";
                amplitude3.Text = (3 < e.Count) ? e.Amplitudes[3].ToString() : "";
                amplitude4.Text = (4 < e.Count) ? e.Amplitudes[4].ToString() : "";
                amplitude5.Text = (5 < e.Count) ? e.Amplitudes[5].ToString() : "";
                amplitude6.Text = (6 < e.Count) ? e.Amplitudes[6].ToString() : "";
                amplitude7.Text = (7 < e.Count) ? e.Amplitudes[7].ToString() : "";
            }
            catch (PhidgetException ex)
            { errorBox.addMessage(ex.ToString()); }
        }

        private void sonarQuietModeBox_CheckedChanged(object sender, EventArgs e)
        {
            try
            {
                distance.SonarQuietMode = sonarQuietModeBox.Checked;
            }
            catch (Exception ex)
            {
                errorBox.addMessage("Error setting sonar quiet mode: " + ex.Message);
            }
        }

        private void dataIntervalTrk_LogScroll(object sender, EventArgs e)
        {
            if (MouseButtons != MouseButtons.Left)
            {
                try
                {
                    distance.DataInterval = (int)dataIntervalTrk.Value;
                }
                catch (Exception ex)
                {
                    errorBox.addMessage("Error setting data interval: " + ex.Message);
                }
            }
        }

        private void dataIntervalTrk_LogMouseUp(object sender, MouseEventArgs e)
        {
            try
            {
                distance.DataInterval = (int)dataIntervalTrk.Value;
            }
            catch (Exception ex)
            {
                errorBox.addMessage("Error setting data interval: " + ex.Message);
            }
        }

        private void changeTriggerTrk_LogScroll(object sender, EventArgs e)
        {
            if (MouseButtons != MouseButtons.Left)
            {
                try
                {
                    distance.DistanceChangeTrigger = (int)changeTriggerTrk.Value;
                }
                catch (Exception ex)
                {
                    errorBox.addMessage("Error setting change trigger: " + ex.Message);
                }
            }
        }

        private void changeTriggerTrk_LogMouseUp(object sender, MouseEventArgs e)
        {
            try
            {
                distance.DistanceChangeTrigger = (int)changeTriggerTrk.Value;
            }
            catch (Exception ex)
            {
                errorBox.addMessage("Error setting change trigger: " + ex.Message);
            }
        }
    }
}